@@ -8,6 +8,7 @@ class TaskTree(object):
 
     def __init__(self):
         self.children = []
+        self.last_node = self
 
     def add_task(self, func, args=None, kwargs=None):
         if args is None:
@@ -18,6 +19,17 @@ class TaskTree(object):
         self.children.append(node)
         return node
 
+    def push(self, func, args=None, kwargs=None):
+        self.last_node = self.last_node.add_task(func, args, kwargs)
+        return self.last_node
+
+    def pop(self):
+        if self.last_node == self:
+            raise IndexError('pop from empty stack')
+        parent = self.last_node.parent
+        parent.children.remove(self.last_node)
+        self.last_node = parent
+
     def apply_async(self):
         tasks = []
         for node in self.children:
@@ -36,10 +48,27 @@ class TaskTree(object):
         result = taskset.apply_async()
         return result
 
+    def apply_and_join(self):
+        """ Execute tasks asynchronously and wait for the latest result.
+
+        Method can be useful in conjunction with pop()/push() methods. In such
+        a case method returns a list of results in the order which corresponds
+        to the order of nodes being pushed.
+        """
+        output = []
+        first_result = self.apply_async().join()[0]
+        while True:
+            output.append(first_result)
+            if not getattr(first_result, 'async_result', None):
+                break
+            first_result = first_result.async_result.join()[0]
+        return output
+
 
 class TaskTreeNode(object):
 
     def __init__(self, func, args=None, kwargs=None):
+        self.parent = None
         if args is None:
             args = []
         if kwargs is None:
@@ -56,6 +85,7 @@ class TaskTreeNode(object):
             kwargs = {}
         node = TaskTreeNode(func, args, kwargs)
         self.children.append(node)
+        node.parent = self
         return node
 
     def _get_child_tasks(self):
